-- ============================================================================
-- == HalonaTools - A collection of tools required by members of the         ==
-- ==               Alliance PVE Raiding Guild "Halona" of the Silvermoon EU ==
-- ==               Realm of Blizzard Entertainment's "World of Warcraft"    ==
-- ==               MMORPG. These tools are provided in the form of an addon ==
-- ==               that supplements those controls found within the default ==
-- ==               UI provided by the game and those provided by the        ==
-- ==               authors of other third-party UI addons.                  ==
-- ==                                                                        ==
-- ==               This addon is based upon the Ace3 library framework and  ==
-- ==               additionally supports other standards such as the LDB    ==
-- ==               (aka: LibDataBroker) method for enabling integration of  ==
-- ==               this addon with a wide-range of LDB-compatible "Display" ==
-- ==               addons.                                                  ==
-- ==                                                                        ==
-- ==               Development of this addon is managed via the resources   ==
-- ==               provided by the WoWAce community as follows.             ==
-- ==                                                                        ==
-- ==               Home-Page: http://www.wowace.com/projects/HalonaTools    ==
-- ==               SVN......: svn://svn.wowace.com/projects/halonatools     ==
-- ==               Download.: http://wow.curse.com/addons/wow/HalonaTools   ==
-- ==               Author...: broliant@broliant.com                         ==
-- ============================================================================

HalonaTools   = LibStub("AceAddon-3.0"):NewAddon("HalonaTools", "AceConsole-3.0", "AceEvent-3.0")
local L       = LibStub("AceLocale-3.0"):GetLocale("HalonaTools")

-- ============================================================================
-- == OnInitialize()                                                         ==
-- ==                                                                        ==
-- == This function is called when the addon is initialised.                 ==
-- ============================================================================

function HalonaTools:OnInitialize()
end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is enabled.                     ==
-- ============================================================================

function HalonaTools:OnEnable()

  HTConsole:BroadcastMessageAVB()                    -- Addon Version Broadcast

end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is disabled.                    ==
-- ============================================================================

function HalonaTools:OnDisable()
end

-- ============================================================================
-- == DoHeartbeat()                                                          ==
-- ==                                                                        ==
-- == This function is intended to provide a place where non-critical tasks  ==
-- == that need to be performed periodically can be called from. Typically   ==
-- == each HT component may have an instance of this method and each will    ==
-- == be called in turn by the corresponding function in HTConsole which is  ==
-- == considered to be the master-heartbeat component.                       ==
-- ============================================================================

function HalonaTools:DoHeartbeat()

  HalonaTools:GuildAdminChecks()

end

-- ============================================================================
-- == BeginRaidInvitationPeriod(characterName)                               ==
-- ==                                                                        ==
-- == This function handles the initiation of the raid invitation period.    ==
-- == Specifically, it sends an announcement to the GUILD chat channel and   ==
-- == updates the GMOTD to indicate that the period is in effect and to      ==
-- == advise members who to contact for an invitation to the raid.           ==
-- ==                                                                        ==
-- == Additionally, a broadcast message is sent out via the addon channel so ==
-- == that other players using HalonaTools may detect the event and offer    ==
-- == the player the option of requesting an invitation automatically.       ==
-- ============================================================================

function HalonaTools:BeginRaidInvitationPeriod(characterName)

  local gmotdText = string.gsub(L["ht-raid-invites-begin-gmotd-01"], "CHARACTER_NAME", characterName)
  local gchatText = string.gsub(L["ht-raid-invites-begin-gchat-01"], "CHARACTER_NAME", characterName)

  if HTCommon:IsOfficer(characterName) then
    GuildSetMOTD(gmotdText)
  end

  HTConsole:SendTitleBar("GUILD")
  HTConsole:SendWrappedMessage(L["ht-raid-invites-begin-title-01"], "GUILD", nil, nil)
  HTConsole:SendTitleBar("GUILD")
  HTConsole:SendWrappedMessage(gchatText, "GUILD", nil, nil)
  HTConsole:SendTitleBar("GUILD")

  --
  -- Send the raid invitation announcement addon message.
  --
  local strCharacterName = HTCommon:FindPlayerByCharacterName(characterName)
  local simpleName       = HTCommon:GetSimpleCharacterName(strCharacterName)

  SendAddonMessage(L["con-addonmessage-prefix-htria"], simpleName, "GUILD")

end

-- ============================================================================
-- == BeginRaidSession(characterName)                                        ==
-- ==                                                                        ==
-- == This function handles the initiation of the raid session.              ==
-- == Specifically, it sends an announcement to the GUILD chat channel and   ==
-- == updates the GMOTD to indicate that the raid is in effect and to        ==
-- == advise members who to contact for to join the raid queue.              ==
-- ==                                                                        ==
-- == Additionally, a broadcast message is sent out via the addon channel so ==
-- == that other players using HalonaTools may detect the event and offer    ==
-- == the player the option of requesting a queue spot automatically.        ==
-- ============================================================================

function HalonaTools:BeginRaidSession(characterName)

  local gmotdText = string.gsub(L["ht-raid-session-begin-gmotd-01"], "CHARACTER_NAME", characterName)
  local gchatText = string.gsub(L["ht-raid-session-begin-gchat-01"], "CHARACTER_NAME", characterName)

  if HTCommon:IsOfficer(characterName) then
    GuildSetMOTD(gmotdText)
  end

  HTConsole:SendTitleBar("GUILD")
  HTConsole:SendWrappedMessage(L["ht-raid-session-begin-title-01"], "GUILD", nil, nil)
  HTConsole:SendTitleBar("GUILD")
  HTConsole:SendWrappedMessage(gchatText, "GUILD", nil, nil)
  HTConsole:SendTitleBar("GUILD")

  --
  -- Send the raid invitation announcement addon message.
  --
  local strCharacterName = HTCommon:FindPlayerByCharacterName(characterName)
  local simpleName       = HTCommon:GetSimpleCharacterName(strCharacterName)

  SendAddonMessage(L["con-addonmessage-prefix-htrqa"], simpleName, "GUILD")

end

-- ============================================================================
-- == EndRaidSession(characterName)                                          ==
-- ==                                                                        ==
-- == This function handles the termination of the raid session.             ==
-- ============================================================================

function HalonaTools:EndRaidSession(characterName)

  local gmotdText = string.gsub(L["ht-raid-session-end-gmotd-01"], "CHARACTER_NAME", characterName)
  local gchatText = string.gsub(L["ht-raid-session-end-gchat-01"], "CHARACTER_NAME", characterName)

  if HTCommon:IsOfficer(characterName) then
    GuildSetMOTD(gmotdText)
  end

  HTConsole:SendTitleBar("GUILD")
  HTConsole:SendWrappedMessage(L["ht-raid-session-end-title-01"], "GUILD", nil, nil)
  HTConsole:SendTitleBar("GUILD")
  HTConsole:SendWrappedMessage(gchatText, "GUILD", nil, nil)
  HTConsole:SendTitleBar("GUILD")

  --
  -- Send the raid invitation announcement addon message.
  --
  local strCharacterName = HTCommon:FindPlayerByCharacterName(characterName)
  local simpleName       = HTCommon:GetSimpleCharacterName(strCharacterName)

  SendAddonMessage(L["con-addonmessage-prefix-htrea"], simpleName, "GUILD")

end

-- ============================================================================
-- == JoinGuildRaidEvent()                                                   ==
-- ==                                                                        ==
-- == This function examines the GMOTD to determine whether a guild raid     ==
-- == event is either about to take place or currently taking place and      ==
-- == places a request on the user's behalf accordingly if there is.         ==
-- ============================================================================

function HalonaTools:JoinGuildRaidEvent()

  if HTCommon:IsPlayerInParty() or HTCommon:IsPlayerInRaid() then
    HTConsole:SendChatMessage(L["ht-raid-check-skipped"] , "CHAT", nil, nil)
    return
  end

  -- == -----------------------------------------------------------------------
  -- == -- Retrieve the GMOTD, decode into component parts. (ie: Words.)     --
  -- == -----------------------------------------------------------------------
  local gmotdActualText     = HTData:SafeString(GetGuildRosterMOTD())
  local gmotdActualWordList = HTData:StringSplit(gmotdActualText, " ")
  local gmotdRIAWordList    = HTData:StringSplit(L["ht-raid-invites-begin-gmotd-01"], " ")
  local gmotdRQAWordList    = HTData:StringSplit(L["ht-raid-session-begin-gmotd-01"], " ")

  local riaMatchText = gmotdRIAWordList[1] .. " " .. gmotdRIAWordList[2] .. " " .. gmotdRIAWordList[3] .. " " .. gmotdRIAWordList[4] .. " " .. gmotdRIAWordList[5] .. " " .. gmotdRIAWordList[6]
  local riaMatchLen  = string.len(riaMatchText)

  local rqaMatchText = gmotdRQAWordList[1] .. " " .. gmotdRQAWordList[2] .. " " .. gmotdRQAWordList[3] .. " " .. gmotdRQAWordList[4] .. " " .. gmotdRQAWordList[5]
  local rqaMatchLen  = string.len(rqaMatchText)

  -- == -----------------------------------------------------------------------
  -- == -- Test the GMOTD: Is the raid invitation period open?               --
  -- == -----------------------------------------------------------------------
  if string.len(gmotdActualText) > riaMatchLen then
    if string.sub(gmotdActualText, 1, riaMatchLen) == riaMatchText then
      HTGUI:PopupAskRequestRaidInvitation(HTData:SafeString(gmotdActualWordList[12]), UnitName("player"))
      return
    end
  end

  -- == -----------------------------------------------------------------------
  -- == -- Test the GMOTD: Is a raid currently in progress?                  --
  -- == -----------------------------------------------------------------------
  if string.len(gmotdActualText) > rqaMatchLen then
    if string.sub(gmotdActualText, 1, rqaMatchLen) == rqaMatchText then
      HTGUI:PopupAskRequestRaidQueueListing(HTData:SafeString(gmotdActualWordList[12]), UnitName("player"))
      return
    end
  end

  -- No raid event in progress, report then return.
  HTConsole:SendChatMessage(L["ht-raid-not-in-progress-01"], "CHAT", nil, nil)


end

-- ============================================================================
-- == CreateDBMTimer(duration, name, broadcastToRaid)                        ==
-- ==                                                                        ==
-- == This function creates a new DBM (Deadly Boss-Mods) timer where the     ==
-- == duration, name and whether the timer is to be broadcast to the raid    ==
-- == are provided as parameters.                                            ==
-- ============================================================================

function HalonaTools:CreateDBMTimer(duration, name, broadcastToRaid)

  DBM:CreatePizzaTimer(duration, name, broadcastToRaid)

end

-- ============================================================================
-- == ShowVersions()                                                         ==
-- ==                                                                        ==
-- == This function simply displays the versions of HT currently being used  ==
-- == by those currently online.                                             ==
-- ============================================================================

function HalonaTools:ShowVersions()

  local currentVersions = HTConsole:GetVersions() -- HTConfig:GetVersions(nil)
  local myVersionName   = L["addon-version-name"]
  local myVersionNumber = L["addon-version-number"]

  HTConsole:SendChatMessage("[HT] - Version Check - Guild Members.", "CHAT", nil, nil)

  for i,v in ipairs(currentVersions) do

    local args = HTData:StringSplit(v, "::")

    local currentUsername      = args[1]
    local currentVersionName   = args[2]
    local currentVersionNumber = args[3]

    local messageText = "|c0ffffff00" .. currentUsername .. "|r"

    messageText = messageText .. " - |cff00c0ff" .. currentVersionName .. "|r:"

    if currentVersionNumber < myVersionNumber then
      messageText = messageText .. "|c0ffff0000" .. currentVersionNumber .. "|r"
    elseif currentVersionNumber > myVersionNumber then
      messageText = messageText .. "|c0ff00ff00" .. currentVersionNumber .. "|r"
    else
      messageText = messageText .. "|c0ffffff00" .. currentVersionNumber .. "|r"
    end

    HTConsole:SendChatMessage(messageText, "CHAT", nil, nil)

  end
end

-- ============================================================================
-- == ShowReportUsageStatus(channel)                                         ==
-- ==                                                                        ==
-- == This function displays a report of HT's usage status within the        ==
-- == members of the guild currently online.                                 ==
-- ============================================================================

function HalonaTools:ShowReportUsageStatus(channel)

  -- Fetch all data required for report.
  local userDataTypeCurrent = HTConsole:GetHTUserData(L["con-ht-userdata-type-current"])
  local userDataTypeOld     = HTConsole:GetHTUserData(L["con-ht-userdata-type-old"])
  local userDataTypeAny     = HTConsole:GetHTUserData(L["con-ht-userdata-type-any"])
  local userDataTypeNone    = HTConsole:GetHTUserData(L["con-ht-userdata-type-none"])
  local maxVersionName      = HTConsole:GetMaxVersionName()
  local maxVersionNumber    = HTConsole:GetMaxVersionNumber()
  local goodVersionText     = HTData:BuildStringFromTable(userDataTypeCurrent, L["ht-usage-status-uptodate"],  ", ")
  local oldVersionText      = HTData:BuildStringFromTable(userDataTypeOld,     L["ht-usage-status-outofdate"], ", ")
  local noVersionText       = HTData:BuildStringFromTable(userDataTypeNone,    L["ht-usage-status-missing"],   ", ")
  local maxVersionText      = L["ht-usage-status-maxversion"]
  maxVersionText            = string.gsub(maxVersionText, "VERSION_NAME", maxVersionName)
  maxVersionText            = string.gsub(maxVersionText, "VERSION_NUMBER", maxVersionNumber)

  -- Report Usage Status to specified channel.
  HTConsole:SendTitleBar(channel)
  HTConsole:SendWrappedMessage(L["ht-usage-status-report-title"], channel, nil, nil)
  HTConsole:SendTitleBar(channel)
  HTConsole:SendWrappedMessage(maxVersionText, channel, nil, nil)

  -- Only output each of the following if there are users contained in the table.
  if #userDataTypeCurrent > 0 then HTConsole:SendWrappedMessage(goodVersionText, channel, nil, nil) end
  if #userDataTypeOld     > 0 then HTConsole:SendWrappedMessage(oldVersionText,  channel, nil, nil) end
  if #userDataTypeNone    > 0 then HTConsole:SendWrappedMessage(noVersionText,   channel, nil, nil) end

  HTConsole:SendTitleBar(channel)

end

-- ============================================================================
-- == GuildAdminChecks()                                                     ==
-- ==                                                                        ==
-- == This function is responsible for running the guild admin checks that   ==
-- == correspond to the player's rank.                                       ==
-- ============================================================================

function HalonaTools:GuildAdminChecks()

  HalonaTools:GuildAdminOfficerChecks()

end

-- ============================================================================
-- == GuildAdminOfficerChecks()                                              ==
-- ==                                                                        ==
-- == This function is responsible for running the guild admin checks that   ==
-- == correspond to officer ranked players.                                  ==
-- ============================================================================

function HalonaTools:GuildAdminOfficerChecks()

  -- HTConsole:Debug(UnitName("player").." officer status: ["..HTData:BooleanAsText(HTCommon:IsOfficer(UnitName("player"))).."]")
  
  -- Only proceed past this point if the player is an officer of the guild.
  if not HTCommon:IsOfficer(UnitName("player")) then return end

  HTConsole:Debug("Performing HalonaTools:GuildAdminOfficerChecks()...")
  
  -- Check trial member officer notes for conformity. ("Trial yyyy.mm.dd")
  HalonaTools:GuildAdminCheckTrialMemberOfficerNotes()

end

-- ============================================================================
-- == CheckTrialCharacterOfficerNotes()                                      ==
-- ==                                                                        ==
-- == This function checks that all trial ranked characters have their       ==
-- == officer note field set correctly. The proper format for the officer    ==
-- == note of a trial ranked member is as follows:                           ==
-- ==                                                                        ==
-- == Trial dd.mm.yyyy                                                       ==
-- ==                                                                        ==
-- ============================================================================

function HalonaTools:GuildAdminCheckTrialMemberOfficerNotes()

  local totalNumberOfGuildMembers = GetNumGuildMembers(true)
  local problemMemberList         = {}

  for index=1, totalNumberOfGuildMembers, 1 do
    local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(index)
    if HTCommon:IsMainRankedCharacter(name) then
      if HTCommon:IsTrialRankedCharacter(name) then
        if not HalonaTools:IsTrialMemberOfficerNoteValid(officernote) then
          table.insert(problemMemberList.."=["..officernote.."]", name)
        end
      end
    end
  end

  if #problemMemberList > 0 then
    HTConsole:SendChatMessage(L["ht-admin-trial-officer-notes-title"], "CHAT", nil, nil)
    HTConsole:SendChatMessage(L["ht-admin-trial-officer-notes-test"], "CHAT", nil, nil)
    HTConsole:SendChatMessage(L["ht-admin-trial-officer-notes-expected"], "CHAT", nil, nil)
    HTConsole:SendWrappedMessage(HTData:BuildStringFromTable(problemMemberList, L["ht-admin-trial-officer-notes-failures"], ", "), "CHAT", nil, nil)
    HTConsole:SendChatMessage(L["ht-admin-trial-officer-notes-alert"], "ERRORFRAME", nil, nil)
  end

end

-- ============================================================================
-- == IsTrialMemberOfficerNoteValid(officerNote)                             ==
-- ==                                                                        ==
-- == Assumption: New guild members are given the rank of "Trial" and their  ==
-- == officer note is set in the following manner:                           ==
-- ==                                                                        ==
-- == Trial yyyy.mm.dd                                                       ==
-- == 1234567890123456                                                       ==
-- ==                                                                        ==
-- == This function checks the supplied officer-note value and returns true  ==
-- == if it matches the expected format or false otherwise.                  ==
-- ============================================================================

function HalonaTools:IsTrialMemberOfficerNoteValid(officerNote)

   -- Ensure officerNote has a value.
   if officerNote == nil then return false end

   -- Ensure officerNote is exactly the right length.
   if string.len(officerNote) ~= 16 then return false end

   -- Ensure beginning matches "Trial "
   if string.sub(officerNote, 1, 6) ~= "Trial " then return false end

   -- Ensure position 11 is a period.
   if string.sub(officerNote, 11, 11) ~= "." then return false end

   -- Ensure position 14 is a period.
   if string.sub(officerNote, 14, 14) ~= "." then return false end

   -- Ensure positions 7-10 are numerals.
   if not tonumber(string.sub(officerNote, 7,10)) then return false end

   -- Ensure positions 12-13 are numerals.
   if not tonumber(string.sub(officerNote, 12,13)) then return false end

   -- Ensure positions 15-16 are numerals.
   if not tonumber(string.sub(officerNote, 15,16)) then return false end

   dateNumber  = tonumber(string.sub(officerNote, 15,16))
   monthNumber = tonumber(string.sub(officerNote, 12,13))
   yearNumber  = tonumber(string.sub(officerNote,  7,10))

   -- Ensure dateNumber is in valid date-range.
   if dateNumber < 0 or dateNumber > 31 then return false end

   -- Ensure monthNumber is in valid date-range.
   if monthNumber < 0 or monthNumber > 12 then return false end

   -- Ensure yearNumber is in valid date-range.
   if yearNumber < 0 or yearNumber > 2000000 then return false end

   -- All tests passed, officerNote is valid!
   return true

end
